<nz-modal [(nzVisible)]="isVisible" nzTitle="Select a Position"
          [nzFooter]="null" [nzClosable]="false">
    <ul nz-list nzBordered nzSize="large">
        <li nz-list-item *ngFor="let item of positions" nzNoFlex
            (click)="handlePositionSelected(item)">
            {{ item.coordSet.name }} -{{item.position.x}}x{{item.position.y}}
        </li>
    </ul>
</nz-modal>
