@import "~@synerty/peek-plugin-base-js/src/styles/_mixins";
@import "~@synerty/peek-plugin-base-js/src/styles/_variables";

.peek-tabs-section {
    width: unset;
}

.peek-tabs-section-body {
    padding: $general_padding_1;
    top: $title_bar_height;
}
